package de.erethon.hephaestus;

import com.destroystokyo.paper.event.player.PlayerArmorChangeEvent;
import de.erethon.spellbook.api.SpellCastEvent;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.craftbukkit.v1_20_R3.inventory.CraftItemStack;
import org.bukkit.entity.Item;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockDropItemEvent;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityDropItemEvent;
import org.bukkit.event.entity.EntityPickupItemEvent;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.event.player.PlayerAttemptPickupItemEvent;
import org.bukkit.event.player.PlayerDropItemEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.server.ServerLoadEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.java.JavaPlugin;

import static net.minecraft.server.MinecraftServer.LOGGER;

public class HBukkitHandler implements Listener {

    private final Plugin plugin;
    private final ItemLibrary lib;

    public HBukkitHandler(JavaPlugin pl, ItemLibrary library) {
        this.lib = library;
        this.plugin = pl;
        Bukkit.getPluginManager().registerEvents(this, plugin);
        LOGGER.info("Initialised HBukkitHandler using " + plugin.getName());
    }

    @EventHandler
    private void onInvClick(InventoryClickEvent event) {
        if (event.getCurrentItem() == null) {
            return;
        }
        HItemStack item = ItemLibrary.getHItemFromBukkit(event.getCurrentItem());
        if (item != null && item.getBehaviour() != null) {
            item.getBehaviour().onInventoryClick(CraftItemStack.unwrap(event.getCurrentItem()), (Player) event.getWhoClicked(), event.getClick(), event.getAction(), event);
        }
    }

    @EventHandler
    private void onDamage(EntityDamageByEntityEvent event) {
        if (event.getEntity() instanceof LivingEntity living && living.getEquipment() != null) {
            ItemStack mainHand = living.getEquipment().getItemInMainHand();
            HItemStack item = ItemLibrary.getHItemFromBukkit(mainHand);
            if (item != null && item.getBehaviour() != null) {
                item.getBehaviour().onDamaged(CraftItemStack.unwrap(mainHand), living, event);
            }
            ItemStack offHand = living.getEquipment().getItemInOffHand();
            item = ItemLibrary.getHItemFromBukkit(mainHand);
            if (item != null && item.getBehaviour() != null) {
                item.getBehaviour().onDamaged(CraftItemStack.unwrap(offHand), living, event);
            }
            for (ItemStack armor : living.getEquipment().getArmorContents()) {
                if (armor == null) {
                    continue;
                }
                item = ItemLibrary.getHItemFromBukkit(armor);
                if (item != null && item.getBehaviour() != null) {
                    item.getBehaviour().onDamaged(CraftItemStack.unwrap(armor), living, event);
                }
            }
        }
    }

    @EventHandler
    private void onDamage(SpellCastEvent event) {
        LivingEntity living = event.getCaster();
        if (living.getEquipment() != null) {
            ItemStack mainHand = living.getEquipment().getItemInMainHand();
            HItemStack item = ItemLibrary.getHItemFromBukkit(mainHand);
            if (item != null && item.getBehaviour() != null) {
                item.getBehaviour().onSpellCast(CraftItemStack.unwrap(mainHand), living, event);
            }
            ItemStack offHand = living.getEquipment().getItemInOffHand();
            item = ItemLibrary.getHItemFromBukkit(mainHand);
            if (item != null && item.getBehaviour() != null) {
                item.getBehaviour().onSpellCast(CraftItemStack.unwrap(offHand), living, event);
            }
            for (ItemStack armor : living.getEquipment().getArmorContents()) {
                if (armor == null) {
                    continue;
                }
                item = ItemLibrary.getHItemFromBukkit(armor);
                if (item != null && item.getBehaviour() != null) {
                    item.getBehaviour().onSpellCast(CraftItemStack.unwrap(armor), living, event);
                }
            }
        }
    }

    @EventHandler
    private void onPlayerDrop(PlayerDropItemEvent event) {
        ItemStack stack = event.getItemDrop().getItemStack();
        HItemStack item = ItemLibrary.getHItemFromBukkit(stack);
        if (item != null && item.getBehaviour() != null) {
            item.getBehaviour().onPlayerDrop(CraftItemStack.unwrap(stack), event.getPlayer(), event);
        }
    }

    @EventHandler
    private void onEntityDrop(EntityDropItemEvent event) {
        ItemStack stack = event.getItemDrop().getItemStack();
        HItemStack item = ItemLibrary.getHItemFromBukkit(stack);
        if (item != null && item.getBehaviour() != null) {
            item.getBehaviour().onEntityDrop(CraftItemStack.unwrap(stack), event.getEntity(), event);
        }
    }

    @EventHandler
    private void onBlockDrop(BlockDropItemEvent event) {
        for (Item stack : event.getItems()) {
            HItemStack item = ItemLibrary.getHItemFromBukkit(stack.getItemStack());
            if (item != null && item.getBehaviour() != null) {
                item.getBehaviour().onBlockDrop(CraftItemStack.unwrap(stack.getItemStack()), event.getPlayer(), event);
            }
        }
    }

    @EventHandler
    private void onPickup(PlayerAttemptPickupItemEvent event) {
        ItemStack stack = event.getItem().getItemStack();
        HItemStack item = ItemLibrary.getHItemFromBukkit(stack);
        if (item != null && item.getBehaviour() != null) {
            item.getBehaviour().onPickup(CraftItemStack.unwrap(stack), event.getPlayer(), event);
        }
    }

    @EventHandler
    private void onEntityPickup(EntityPickupItemEvent event) {
        ItemStack stack = event.getItem().getItemStack();
        HItemStack item = ItemLibrary.getHItemFromBukkit(stack);
        if (item != null && item.getBehaviour() != null) {
            item.getBehaviour().onEntityPickup(CraftItemStack.unwrap(stack), event.getEntity(), event);
        }
    }

    @EventHandler
    private void onInteract(PlayerInteractEvent event) {
        if (!event.hasItem()) {
            return;
        }
        HItemStack item = ItemLibrary.getHItemFromBukkit(event.getItem());
        if (event.getAction().isRightClick()) {
            if (item != null && item.getBehaviour() != null) {
                item.getBehaviour().onRightClick(CraftItemStack.unwrap(event.getItem()), event.getPlayer(), event);
            }
        }
        if (event.getAction().isLeftClick()) {
            if (item != null && item.getBehaviour() != null) {
                item.getBehaviour().onLeftClick(CraftItemStack.unwrap(event.getItem()), event.getPlayer(), event);
            }
        }
    }

    @EventHandler
    private void onEquip(PlayerArmorChangeEvent event) {
        if (event.getNewItem() == event.getOldItem()) {
            return;
        }
        ItemStack stack = event.getNewItem();
        HItemStack item = ItemLibrary.getHItemFromBukkit(stack);
        if (item != null && item.getBehaviour() != null && stack.getType() != Material.AIR) {
            item.getBehaviour().onEquip(CraftItemStack.unwrap(stack), event.getPlayer(), event);
            return;
        }
        stack = event.getOldItem();
        item = ItemLibrary.getHItemFromBukkit(stack);
        if (item != null && item.getBehaviour() != null) {
            item.getBehaviour().onUnequip(CraftItemStack.unwrap(stack), event.getPlayer(), event);
        }
    }

    @EventHandler
    private void onServerStartFinish(ServerLoadEvent event) {
        lib.readyBehaviours();
    }

}
