package de.erethon.hephaestus;

import io.papermc.paper.adventure.PaperAdventure;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.minimessage.MiniMessage;
import net.minecraft.core.registries.BuiltInRegistries;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.Main;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.craftbukkit.v1_20_R3.util.CraftMagicNumbers;
import org.bukkit.plugin.Plugin;
import org.checkerframework.checker.nullness.qual.Nullable;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;

import static net.minecraft.server.MinecraftServer.LOGGER;

public class HItem extends YamlConfiguration {

    private final MiniMessage mm = MiniMessage.miniMessage();

    private ResourceLocation id;
    private Item baseItem;
    private HItemBehaviour behaviour;
    private Component displayName;
    private List<Component> lore;
    private String mojangID;
    private int modelData;
    private Map<EquipmentSlot, AttributeModifier> modifierMap;
    private int maxStackSize;
    private int maxDurability;
    private String classURL;
    private boolean hasBehaviour = false;
    private String pluginName;
    private Plugin plugin;
    private Class<? extends HItemBehaviour> behaviourClass;
    private ClassLoader classLoader;

    private boolean isEnabled = true;
    private HItemStack replaceWith = null;

    public HItem(Plugin plugin) {
    }

    public HItem(File file) {
        super();
        try {
            load(file);
        } catch (IOException | InvalidConfigurationException e) {
            Bukkit.getLogger().log(Level.WARNING, "[Hephaestus] Failed to load " + file.getName());
            Bukkit.getLogger().log(Level.WARNING, e.getMessage());
            throw new RuntimeException(e);
        }
    }

    public ResourceLocation getId() {
        return id;
    }

    public void setId(ResourceLocation location) {
        this.id = location;
    }

    public Component getDisplayName() {
        return displayName;
    }

    public List<Component> getLore() {
        return lore;
    }

    public String getMojangID() {
        return mojangID;
    }

    public void setMojangID(String mojangID) {
        this.mojangID = mojangID;
    }

    public Item getBaseItem() {
        return baseItem;
    }

    public void setClassURL(String url) {
        this.classURL = url;
    }

    public int getModelData() {
        return modelData;
    }

    public Map<EquipmentSlot, AttributeModifier> getModifierMap() {
        return modifierMap;
    }

    public int getMaxStackSize() {
        return maxStackSize;
    }

    public int getMaxDurability() {
        return maxDurability;
    }

    public boolean isEnabled() {
        return isEnabled;
    }

    public HItemStack getReplaceWith() {
        return replaceWith;
    }

    public @Nullable String getClassURL() {
        return classURL;
    }

    public String getOwningPlugin() {
        return pluginName;
    }

    public void setOwningPlugin(String name) {
        pluginName = name;
    }

    public boolean hasBehaviour() {
        return hasBehaviour;
    }

    public HItemBehaviour getBehaviour() {
        return behaviour;
    }

    public void setBehaviour(HItemBehaviour behaviour) {
        this.behaviour = behaviour;
    }

    public void setPlugin(Plugin plugin) {
        this.plugin = plugin;
    }

    public HItemStack getItem() {
        return getItem(1);
    }

    public HItemStack getItem(int count) {
        HItemStack item = prepareNewStack().hItemStack;
        item.getStack().setCount(count);
        return item;
    }

    private ItemStack prepareNewStack() {
        ItemStack baseItem = new ItemStack(BuiltInRegistries.ITEM.get(new ResourceLocation(getMojangID())));
        for (EquipmentSlot slot : EquipmentSlot.values()) {
            // baseItem.getAttributeModifiers(slot).clear(); this is immutable apparently
        }
        baseItem.hItemStack = new HItemStack(baseItem, this);
        baseItem.getOrCreateTag().putInt("CustomModelData", getModelData());
        baseItem.getOrCreateTag().putString("HephaestusID", id.toString());
        baseItem.setHoverName(PaperAdventure.asVanilla(displayName));
        baseItem.getBukkitStack().editMeta(m -> m.lore(lore)); // Annoying but oh well.
        return baseItem;
    }

    public void loadBehaviour() {
        if (plugin == null) {
            plugin = Bukkit.getPluginManager().getPlugin(pluginName);
            if (plugin == null) {
                LOGGER.info("Failed to find plugin " + pluginName + " for item " + id.toString());
            }
        }
        classLoader = plugin.getClass().getClassLoader();
        try {
            behaviourClass = ((Class<? extends HItemBehaviour>) Class.forName(getClassURL(), true, classLoader));
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        }
        try {
            behaviour = behaviourClass.getDeclaredConstructor(HItem.class).newInstance(this);
        } catch (InstantiationException | IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
            throw new RuntimeException(e);
        }
        LOGGER.info("Loaded behaviour for " + id + ": " + behaviour.getClass().getName());
        hasBehaviour = true;
    }

    @Override
    public void load(@NotNull File file) throws IOException, InvalidConfigurationException {
        super.load(file);
        id = ResourceLocation.tryParse(getString("id", "minecraft:stone"));
        if (contains("class")) {
            classURL = getString("class");
            pluginName = getString("owningPlugin");
            hasBehaviour = true;
        }
        isEnabled = getBoolean("enabled", true);
        displayName = mm.deserialize(getString("displayName", ""));
        for (String string : getStringList("lore")) {
            lore.add(mm.deserialize(string));
        }
        mojangID = getString("mojangID", "minecraft:stone");
        baseItem = BuiltInRegistries.ITEM.get(new ResourceLocation(mojangID));
        modelData = getInt("modelData", 0);
        maxStackSize = getInt("maxStackSize", 1);
        maxDurability = getInt("maxDurability", 0);
        if (contains("modifiers")) {
            for (String slot : getConfigurationSection("modifiers").getKeys(false)) {
                EquipmentSlot equipmentSlot = EquipmentSlot.byName(slot);
                AttributeModifier modifier = new AttributeModifier(
                    getString("modifiers." + slot + ".name", "HephaestusModifier"),
                    getDouble("modifiers." + slot + ".amount", 0),
                    AttributeModifier.Operation.valueOf(getString("modifiers." + slot + ".operation", "ADD_NUMBER")));
                modifierMap.put(equipmentSlot, modifier);
            }
        }
    }


    public void save(File file) {
        set("id", id.toString());
        set("owningPlugin", pluginName);
        set("class", classURL);
        set("enabled", isEnabled);
        set("mojangID", mojangID);
        try {
            super.save(file);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public static class Builder {
        private final HItem item;
        private final Plugin plugin;
        private HItemBehaviour behaviour;

        public Builder(Plugin plugin, ResourceLocation location) {
            this.item = new HItem(plugin);
            this.plugin = plugin;
            item.setId(location);
        }

        public Builder behaviour(HItemBehaviour behaviour) {
            this.behaviour = behaviour;
            item.setClassURL(behaviour.getClass().getName());
            item.setOwningPlugin(plugin.getName());
            item.setPlugin(plugin);
            return this;
        }
        public Builder baseItem(Material material) {
            Item item1 = CraftMagicNumbers.getItem(material);
            item.setMojangID(BuiltInRegistries.ITEM.getKey(item1).toString());
            return this;
        }

        public HItem register() {
            Main.itemLibrary.registerItem(plugin, item);
            File folder = Main.itemLibrary.getFolder();
            File saveFolder = new File(folder, item.getId().getNamespace());
            if (!saveFolder.exists()) {
                saveFolder.mkdirs();
            }
            File saveFile = new File(saveFolder, item.getId().getPath() + ".yml");
            item.save(saveFile);
            return item;
        }
    }

}
